--[[
The Settings fires two primary events, one for loading (Loaded event) and one for unloading (Saving event).
Any Module can attach to these events and read/write their own values to use.
]]

-- to prevent the number problem (1.0 or 1,0 etc.).
local function flatten(buff, prefix, data)
	for k, v in pairs(data) do
		local keystr = (prefix or "") .. k;

		if type(v) == "table" then
			flatten(buff, keystr .. ".", v);
		else
			buff[keystr] = string.format("%s:%s", type(v), tostring(v));
		end
	end
end

-- https://www.lua.org/pil/14.1.html
local function setfield(t, f, v)
	--for w, d in string.gfind(f, "([%w_]+)(.?)") do
	for w, d in string.gfind(f, "([^%c%z%s\.]+)(\.?)") do
		if d == "." then        -- not last field?
			t[w] = t[w] or {};  -- create table if absent
			t = t[w];           -- get the table
		else                    -- last field
			t[w] = v;           -- do the assignment
		end
	end
end

-- The file name like PocketPluginsSpamFilterSettingsDE.plugindata
Settings = {
	["pluginDataKey"] = (string.gsub(getfenv(1)._.Name, "%.", "")) .. string.upper(Locale.Language),
};

-- Loads the plugin data
Settings.Load = function(self)
	Turbine.PluginData.Load(Turbine.DataScope.Character, self.pluginDataKey, function(data)
		if data == nil then
			return;
		end

		local config = {};

		for k, v in pairs(data) do
			local val;

			if string.sub(v, 1, string.len("number:")) == "number:" then
				val = tonumber(string.sub(v, string.len("number:") + 1));
			elseif string.sub(v, 1, string.len("boolean:")) == "boolean:" then
				val = (string.sub(v, string.len("boolean:") + 1) == tostring(true));
			elseif string.sub(v, 1, string.len("string:")) == "string:" then
				val = string.sub(v, string.len("string:") + 1);
			else
				-- "function:" or whatever
				val = nil;
			end

			setfield(config, k, val);
		end

		Events.FireEvent(self, self.Loaded, config);
	end);
end

-- Fires on plugin data was loaded
Settings.Loaded = function(sender, args)
end

-- Saves the plugin data
Settings.Save = function(self)
	local config = {};
	Events.FireEvent(self, self.Saving, config);

	local data = {};
	flatten(data, "", config);

	Turbine.PluginData.Save(Turbine.DataScope.Character, self.pluginDataKey, data, function(state, errorMessage)
	end);
end

-- Fires before saving plugin data
Settings.Saving = function(sender, args)
end
